//#define USE_SERIAL

#include <Wire.h>
#include "GridEye.h"  // Use local version (patched)
GridEye myeye;


#include <HotKey_GFX.h>    // Core graphics library
#include <HotKey_ST7735.h> // Hardware-specific library
#include <SPI.h>

// Use hardware SPI for the display (for UNO/IteaduinoLite thats sclk = 13 and sid = 11) and pin 10 must be
#define TFT_CS    10
#define TFT_RST    9  // you can also connect this to the Arduino reset in which case, set this #define pin to 0!
#define TFT_DC     8
//#define TFT_SCLK 13 // hardware SPI SCLK=13 (fast)
//#define TFT_MOSI 11 // hardware SPI SDA=11 (fast)
HotKey_ST7735 tft = HotKey_ST7735(TFT_CS,  TFT_DC, TFT_RST);

//#define CLIP_AT_ZERO
#define SHOW_TENS

#define xs (128 / 8)
#define ys (128 / 8) // 160 actually!

#define VALUE_MIN -2047
#define VALUE_MAX 2047
#define VALUE_RANGE (VALUE_MAX-VALUE_MIN)

#define MIN_VALUE_SPAN 15 * 4 // How many degrees to always keep in scale (>= 10 is recommended)
//#define VALUE2C 0.25
int valueMin = -20 *4;  //TEMP_MAX;
int valueMax = 60 *4;  //TEMP_MIN;


uint16_t rgb2565(uint8_t r, uint8_t g, uint8_t b) {
  //return ((r & 0xF8) << 8) | ((g & 0xFC) << 3) | (b >> 3);
  return ((b) >> 3) | (((g) >> 3) << 6) | (((r) >> 3) << 11);
}

uint16_t wheel2565(int wheel) {

  if (wheel < 85) {
   return rgb2565(0, wheel*2, 255 - wheel);
  } else if (wheel < 170) {
   wheel -= 85;
   return rgb2565(wheel*2, 170+wheel*0, 170 - wheel*2);
  } else if (wheel < 255) {
   wheel -= 170; 
   return rgb2565(170+wheel, 170-wheel, 0);
  } else
    return rgb2565(255, 0, 0);

/*
  if (wheel < 64) {
    return rgb2565(0,0,128+wheel*2);
  } else
  if (wheel < 128) {
    wheel -= 64;
    return rgb2565(0, wheel*4, 255-wheel*2);
  } else
  if (wheel < 192) {
    wheel -= 128;
    return rgb2565(wheel*3, 255-wheel, 127-wheel*2);
  } else {
    wheel -= 192;
    return rgb2565(191+wheel, 191-wheel*2, 0);
  }
*/
}

uint16_t value2col(int value) {
  //uint8_t wheel = (temp / 16) + 128;
  //int wheel = map(temp, -80, 80, 0,255);
  int wheel = map(value, valueMin, valueMax, 0,255);
  
  return wheel2565(wheel);
}



void setup(void) {
  Wire.begin(); // I2C for grideye


  // Use this initializer if you're using a 1.8" TFT
  tft.initR(INITR_BLACKTAB);   // initialize a ST7735S chip, black tab

  // Use this initializer (uncomment) if you're using a 1.44" TFT
  //tft.initR(INITR_144GREENTAB);   // initialize a ST7735S chip, black tab
  //tft.fillScreen(ST7735_BLACK);

  //tft.setRotation(2); // Image is rotated by 180
  tft.setRotation(0);
  
  tft.fillScreen(ST7735_RED);

  //myeye.setFramerate(10);

#ifdef USE_SERIAL
  Serial.begin(115200);
  while (!Serial) {
    ; // DUE start up delay
  }
#endif
  tft.fillScreen(ST7735_BLACK);
  /*
  //for(int z = 0; z < 128; z++) {
    for(int y = 0; y < 128; y++) {
      for(int x = 0; x < 128; x++) {
        tft.drawPixel(x, y, rgb2565(x*2,y*2,128));
      }
    }
  //}
  */
  for(int x = 0; x < 128; x++) {
    int v = x*2;
    
    //uint16_t c = wheel2565(v);
    //tft.fillRect(x, 0, 1, 160, c);

    // RGB stripes
    //tft.fillRect(x, 160-16, 1, 6, rgb2565(v,0,0));
    //tft.fillRect(x, 160-10, 1, 5, rgb2565(0,v,0));
    //tft.fillRect(x, 160-5, 1, 5, rgb2565(0,0,v));
    
    tft.fillRect(x, 0, 1, 160, value2col(map(x, 0, 127, valueMin, valueMax)));
  }
  
}


int pixel[64];


void loop(void) {
  int i, x, y, v;
  uint16_t c;

#ifdef USE_SERIAL
  int temp = myeye.thermistorTemp();
  Serial.print(F("Thermistor Temp: "));
  Serial.println(temp * 0.25);
#endif

  // Read all pixels
  myeye.pixelOut(pixel);

  // Display them
  //tft.fillScreen(ST7735_BLACK);
  

  // Find min, max
  int valueMinNew = VALUE_MAX;
  int valueMaxNew = VALUE_MIN;
  for (i = 0; i < 64; i++) {
    v = pixel[i];
    if (v < valueMinNew) valueMinNew = v;
    if (v > valueMaxNew) valueMaxNew = v; 
  }

  // Slowly adjust
  if (valueMinNew > valueMin) valueMin ++;
  else if (valueMinNew < valueMin) valueMin --;
  if (valueMaxNew > valueMax) valueMax ++;
  else if (valueMaxNew < valueMax) valueMax --;

#ifdef CLIP_AT_ZERO
  // Clip at 0 Degrees
  if (valueMin > 0) valueMin = 0;
  if (valueMax < 1) valueMax = 1;
#endif
  //valueMin = valueMinNew;
  //valueMax = valueMaxNew;

  // Keep span
  int valueRange = valueMax - valueMin;
  if (valueRange < MIN_VALUE_SPAN) {
    v = (valueMin+valueMax)/2;
    valueMin = v - MIN_VALUE_SPAN/2;
    valueMax = v + MIN_VALUE_SPAN/2;
  }


  // Draw Scale
  #define SCALESCALE 256
  #define WIDTH 128
  int tOld = SCALESCALE * valueMin/4;
  
  tft.fillRect(0, 0, WIDTH, 16, 0x0000);
  for(x = 0; x < WIDTH; x++) {
    int value = valueMin + (x * valueRange / WIDTH);
    int t = SCALESCALE * value/4;

    //uint16_t c = value2col(value);
    
    if ((tOld <= 0) && (t > 0))
      tft.fillRect(x, 0, 1, 16, rgb2565(0x00,0x00,0xff));
      
    else if ((tOld / (SCALESCALE*10)) < (t / (SCALESCALE*10))) {
      tft.fillRect(x, 4, 1, 12, 0xffff);

      #ifdef SHOW_TENS
        switch (t / (SCALESCALE*10)) {
          case 1: c = rgb2565(0x00, 0xff, 0xff); break;
          case 2: c = rgb2565(0x00, 0xff, 0x00); break;
          case 3: c = rgb2565(0xff, 0xff, 0x00); break;
          case 4: c = rgb2565(0xff, 0x00, 0x00); break;
          default:
            c = rgb2565(0xff, 0xff, 0xff);
        }
        tft.fillRect(x, 0, 2, 4, c);
      #endif
    }
    
    else if ((tOld / (SCALESCALE*5)) < (t / (SCALESCALE*5)))
      tft.fillRect(x, 8, 1, 8, value2col(value));
    
    else if ((tOld / SCALESCALE) < (t / SCALESCALE))
      tft.fillRect(x, 12, 1, 4, value2col(value));
    
    tOld = t;
  }

  
  // Display
  uint8_t x2 = 128-xs;
  uint8_t y2 = 160 - xs - (160 - 128) / 2;
  i = 0;
  for (y = 0; y < 8; y++) {
    x2 = 128-xs;
    for (x = 0; x < 8; x++) {
      //uint16_t c = pixel[i++];
      c = value2col(pixel[i++]);

      //tft.drawPixel(x, y, c);
      tft.fillRect(x2, y2, xs, ys, c);
      x2 -= xs;
    }
    y2 -= ys;
  }
  /*
#ifdef USE_SERIAL
  Serial.println(F("Pixel Output: "));
  for (int i = 0; i < 64; i++) {
    if (i && ((i % 8) == 0)) {
      Serial.println();
    }
    Serial.print(pixel[i]); // 1単位 = 0.25度
    Serial.print(' ');
  }
  Serial.println();
#endif
  */

  
  //delay(500);
  
}
